; **********************************************************************
; **  Compact Flash Identity                    by Stephen C Cousins  **
; **********************************************************************

            .PROC Z80

; Memory map
CodeORG:    .EQU $8000          ;Loader code runs here
LoadAddr:   .EQU $9000          ;Data load address

            .ORG  CodeORG

; Output program details
            LD   DE,About       ;Pointer to error message
            CALL Message        ;Output "Compact flash card info..."
            CALL NewLine        ;Output new line
            CALL NewLine        ;Output new line

; Test if compact flash present
            LD   A,5            ;Tets value for sector count register
            OUT  (CF_SECCOUNT),A  ;Write sector count register
            IN   A,(CF_SECCOUNT)  ;Read sector count register
            CP   5              ;Correct value read back?
            JR   Z,@Present     ;Yes, compact flash is present
            LD   DE,NotFound    ;Pointer to error message
            CALL Message        ;Output message at DE
            CALL NewLine        ;Output new line
            RET                 ;Return to SCMonitor
@Present:

; Get Compact flash identification info
            CALL GetInfo        ;Get CF identification info


; Display results -> Number of sectors on card
            LD   DE,NumSectors
            CALL Message        ;Output "Number of sectors on card: "
            CALL Dollar         ;Output '$'
            LD   DE,(LoadAddr+14)
            CALL HexWord        ;Output most significant word
            LD   DE,(LoadAddr+16)
            CALL HexWord        ;Output least significant word
            CALL NewLine

; Display results -> Card size in Mbytes 
            LD   DE,CardSize
            CALL Message        ;Output "Card size: "
            LD   DE,(LoadAddr+14) ;Number of secotrs hi word
            LD   HL,(LoadAddr+16) ;Number of secotrs lo word
; Determine size in MiB (N x 1,048,576 bytes)
            LD   B,11           ;Number of bits to shift right
@Loop:      SRL  D              ;Shift number of sectors
            RR   E              ;  right by one bit
            RR   H              ;  = divide by 2
            RR   L
            DJNZ @Loop
; HL = Number if MiBytes (N x 1,048,576 bytes)
; Determine number of Megabytes (N x 1,000,000)
; Assume card is 4, 8, 16, 32, 64, 128 or >128 MB
            LD   DE,Above128    ;Default msg ">128 MB"
            LD   A,H            ;Get multiples of 256 MiBytes
            OR   A              ;> 255 MiBytes?
            JR   NZ,@GotSize    ;Yes, so go show "> 128 MB"
            LD   A,L            ;Get number of MiBytes
            CP   128+12         ;Safely > 128 MiBytes?
            JR   NC,@GotSize    ;Yes, so go show "> 128 MB"
            LD   DE,Size128     ;Default msg "128 MB"
            CP   64+6           ;Safely > 64 MiBytes?
            JR   NC,@GotSize    ;Yes, so go show "128 MB"
            LD   DE,Size64      ;Default msg "64 MB"
            CP   32+3           ;Safely > 32 MiBytes?
            JR   NC,@GotSize    ;Yes, so go show "64 MB"
            LD   DE,Size32      ;Default msg "32 MB"
            CP   16+3           ;Safely > 16 MiBytes?
            JR   NC,@GotSize    ;Yes, so go show "32 MB"
            LD   DE,Size16      ;Default msg "16 MB"
            CP   8+2            ;Safely > 8 MiBytes?
            JR   NC,@GotSize    ;Yes, so go show "16 MB"
            LD   DE,Size8       ;Default msg "8 MB"
            CP   4+1            ;Safely > 4 MiBytes?
            JR   NC,@GotSize    ;Yes, so go show "8 MB"
            LD   DE,Size4       ;Default msg "4 MB"
@GotSize:   CALL Message        ;Output size number
            LD   DE,Units
            CALL Message        ;Output " MB"
            CALL NewLine

            CALL NewLine

; Display results -> Model details
            LD   DE,Model
            CALL Message        ;Output "Card model: "
            LD   A,20           ;Number of character pairs
            LD   DE,LoadAddr+54 ;Start of text
            CALL TextSwap       ;Output model text
            CALL NewLine        ;Output new line

; Display results -> Serial number
            LD   DE,Serial
            CALL Message        ;Output "Serial number: "
            LD   A,20           ;Number of characters
            LD   DE,LoadAddr+20 ;Start of text
            CALL Text           ;Output model text
            CALL NewLine        ;Output new line

; Display results -> Firmware version
            LD   DE,Version
            CALL Message        ;Output "Firmware version: "
            LD   A,4            ;Number of character pairs
            LD   DE,LoadAddr+46 ;Start of text
            CALL TextSwap       ;Output model text
            CALL NewLine        ;Output new line

            CALL NewLine

; Display results -> Default number of cylinders
            LD   DE,Cylinders   ;Location of message
            LD   HL,LoadAddr+2  ;Location of data word
            CALL Parameter      ;Output parameter

; Display results -> Default number of heads
            LD   DE,Heads       ;Location of message
            LD   HL,LoadAddr+6  ;Location of data word
            CALL Parameter      ;Output parameter

; Display results -> Default number of sectors per track
            LD   DE,Sectors     ;Location of message
            LD   HL,LoadAddr+12 ;Location of data word
            CALL Parameter      ;Output parameter

; Display results -> Number of unformated bytes per sector
            LD   DE,Bytes       ;Location of message
            LD   HL,LoadAddr+10 ;Location of data word
            CALL Parameter      ;Output parameter


            RET


; **********************************************************************
; **  Messages

About:      .DB  "Compact flash card information v0.1 by Stephen C Cousins",0
Model:      .DB  "Card model:       ",0
Serial:     .DB  "Serial number:    ",0
Version:    .DB  "Firmware version: ",0
NotFound:   .DB  "Compact flash not present",0
NumSectors: .DB  "Number of sectors on card: ",0
CardSize:   .DB  "Card size: ",0
Above128:   .DB  "> 128",0
Size128:    .DB  "128",0
Size64:     .DB  "64",0
Size32:     .DB  "32",0
Size16:     .DB  "16",0
Size8:      .DB  "8",0
Size4:      .DB  "4",0
Units:      .DB  " MB",0
Cylinders:  .DB  "Default number of cylinders:  ",0
Heads:      .DB  "Default number of heads:      ",0
Sectors:    .DB  "Default sectors per track:    ",0
Bytes:      .DB  "Unformatted bytes per sector: ",0



; **********************************************************************
; **  Compact flash functions


; CF registers
CF_DATA     .EQU $10
CF_FEATURES .EQU $11
CF_ERROR    .EQU $11
CF_SECCOUNT .EQU $12
CF_SECTOR   .EQU $13
CF_CYL_LOW  .EQU $14
CF_CYL_HI   .EQU $15
CF_HEAD     .EQU $16
CF_STATUS   .EQU $17
CF_COMMAND  .EQU $17
CF_LBA0     .EQU $13
CF_LBA1     .EQU $14
CF_LBA2     .EQU $15
CF_LBA3     .EQU $16

; CF Features
CF_8BIT     .EQU 1
CF_NOCACHE  .EQU 082H

; CF Commands
CF_RD_SEC   .EQU 020H
CF_WR_SEC   .EQU 030H
CF_IDENTIFY .EQU 0ECH
CF_SET_FEAT .EQU 0EFH


; Read Compact Flash identification info to LoadAddr
GetInfo:    CALL Wait           ;Wait for compact flash to be ready
            LD   A,CF_8BIT      ;Set IDE to be 8bit
            OUT  (CF_FEATURES),A  ;Store feature code
            LD   A,CF_SET_FEAT  ;Get set features command
            OUT  (CF_COMMAND),A ;Perform set features
            CALL Wait           ;Wait for compact flash to be ready
            LD   A,CF_NOCACHE   ;Set no write cache
            OUT  (CF_FEATURES),A  ;Store feature code
            LD   A,CF_SET_FEAT  ;Get set features command
            OUT  (CF_COMMAND),A ;Perform set features
            CALL Wait           ;Wait for compact flash to be ready
            LD   B,1            ;Number of physical sectors to read
            LD   C,0            ;First sector number
            LD   HL,LoadAddr    ;Code from compact flash loads here
; Read sectors where one sector is 4 x 128 byte blocks = 512 bytes
ReadSects:  LD   A,C            ;Get sector number
            OUT  (CF_LBA0),A    ;Set sector number
            XOR  A              ;Set up LBA parameters...
            OUT  (CF_LBA1),A
            OUT  (CF_LBA2),A
            LD   A,0E0H
            OUT  (CF_LBA3),A
            LD   A,1            ;Get number if sectors to read
            OUT  (CF_SECCOUNT),A  ;Store sector count
            LD   A,CF_IDENTIFY  ;Get read sectors command
            OUT  (CF_COMMAND),A ;Perform sector(s) read
            CALL Wait           ;Wait for compact flash to be ready
@TstDRQ:    IN   A,(CF_STATUS)  ;Read status register
            BIT  3,A            ;Test DRQ flag
            JR   Z,@TstDRQ      ;Low so not ready
            LD   E,4            ;1 sector = 4 x 128 byte blocks
            PUSH BC             ;Preserve sector number and count
            LD   C,CF_DATA      ;Compact flash data register
ReadBlock:  LD   B,128          ;Block size
            INIR                ;(HL)=(C), HL=HL+1, B=B-1, repeat
            DEC  E              ;Decrement block counter
            JR   NZ,ReadBlock   ;Repeat until all blocks read
            POP  BC             ;Preserve sector number and count
            INC  C              ;Increment sector number
            DJNZ ReadSects      ;Repeat for all required sectors
            RET


; Wait until compact flash is ready
Wait:
@TstBusy:   IN   A,(CF_STATUS)  ;Read status register
            BIT  7,A            ;Test Busy flag
            JR   NZ,@TstBusy    ;High so busy
@TstReady:  IN   A,(CF_STATUS)  ;Read status register
            BIT  6,A            ;Test Ready flag
            JR   Z,@TstBusy     ;Low so not ready
            RET


; **********************************************************************
; **  API functions


; Output ASCII character A
Character:  PUSH AF
            PUSH BC
            PUSH DE
            LD   C,2            ;API 0x02
            RST  0x30           ;  = Output character
            POP  DE
            POP  BC
            POP  AF
            RET


; Output message at DE
Message:    PUSH AF
            PUSH BC
            PUSH DE
            LD   C,6            ;API 0x06
            RST  0x30           ;  = Output string
            POP  DE
            POP  BC
            POP  AF
            RET


; **********************************************************************
; **  Support functions


; Output new line
NewLine:    PUSH AF
            LD   A,0x0D
            CALL Character      ;Output CR
            LD   A,0x0A
            CALL Character      ;Output LF
            POP  AF
            RET


; Output dollar
Dollar:     PUSH AF
            LD   A,'$'
            CALL Character      ;Output '$'
            POP  AF
            RET
            

; Convert byte (A) to nibbles (DE)
ConvertByteToNibbles:
            PUSH AF
            LD   E,A            ;Get byte to convert
            RRA                 ;Shift top nibble to
            RRA                 ;  botom four bits..
            RRA
            RRA
            AND  0x0F           ;Mask off unwanted bits
            LD   D,A            ;Store top nibble
            LD   A,E            ;Get byte to convert
            AND  0x0F           ;Mask off unwanted bits
            LD   E,A            ;Store bottom nibble
            POP  AF
            RET


; Output hex nibble (A)
HexNibble:  PUSH AF
            AND  0x0F           ;Mask off nibble
            CP   0x0A           ;Nibble > 10 ?
            JR   C,@Skip        ;No, so skip
            ADD  A,7            ;Yes, so add 7
@Skip:      ADD  A,0x30         ;Add ASCII '0'
            CALL Character      ;Write character
            POP  AF
            RET


; Output hex byte (A)
HexByte:    PUSH AF
            PUSH DE
            CALL ConvertByteToNibbles
            LD   A,D
            CALL HexNibble      ;Output most significant nibble
            LD   A,E
            CALL HexNibble      ;Output least significant nibble
            POP  DE
            POP  AF
            RET


; Output hex word (DE)
HexWord:    PUSH AF
            PUSH DE
            LD   A,D
            CALL HexByte        ;Output most significant byte
            LD   A,E
            CALL HexByte        ;Output least significant byte
            POP  DE
            POP  AF
            RET


; Output parameter (string plus hex word)
Parameter   PUSH HL
            CALL Message        ;Output message
            CALL Dollar         ;Output '$'
            POP  HL
            LD   E,(HL)         ;Get parameter address...
            INC  HL
            LD   D,(HL)
            CALL HexWord        ;Output hex parameter value
            CALL NewLine        ;Output new line
            RET


; Output text (at DE) length (A)
Text:       PUSH AF
            PUSH BC
            PUSH DE
            LD   B,A            ;Number of characters 
            LD   A,0x22         ;Quote mark
            CALL Character      ;Output quote mark
@Loop:      LD   A,(DE)         ;Get character from text
            CALL Character      ;Ouptut character
            INC  DE             ;Point to next character
            DJNZ @Loop
            LD   A,0x22         ;Quote mark
            CALL Character      ;Output quote mark
            POP  DE
            POP  BC
            POP  AF
            RET


; Output text (at DE) number of character pairs (A)
; Character order swapped in each word
TextSwap:   PUSH AF
            PUSH BC
            PUSH DE
            LD   B,A            ;Number of characters 
            LD   A,0x22         ;Quote mark
            CALL Character      ;Output quote mark
@Loop:      INC  DE
            LD   A,(DE)         ;Get first character of pair
            DEC  DE
            CALL Character      ;Ouptut character
            LD   A,(DE)         ;Get second character of pair
            CALL Character      ;Ouptut character
            INC  DE             ;Point to next character pair
            INC  DE
            DJNZ @Loop
            LD   A,0x22         ;Quote mark
            CALL Character      ;Output quote mark
            POP  DE
            POP  BC
            POP  AF
            RET

